#!/usr/bin/perl
#
# structure2db.pl version 1.1
#
# A quick* and dirty program to parse the dmoz/ODP structure RDF file.
# Before running this script, the RDF file must be cleaned up by
# removing invalid UTF-8 sequences and illegal XML chars. You can zap
# the bad UTF-8 sequences like so:
#   iconv -c -f UTF-8 -t UTF-8 structure.rdf.u8 > valid.rdf.u8
# You can strip the bad XML with a simple regex like this:
#   s/[\x00-\x08\x0b-\x0c\x0e-\x1f]//g;
# 
# Here's some SQL to creat the tables expected by this program:
#
# CREATE TABLE "structure" (
#   catid INT NOT NULL DEFAULT '0' PRIMARY KEY,
#   topic VARCHAR(1024) NOT NULL,
#   title VARCHAR(1024),
#   description VARCHAR(8192),
#   lastupdate TIMESTAMP
# );
# 
# CREATE TABLE "resources" (
#   catid int NOT NULL DEFAULT '0',
#   rtype VARCHAR(100),
#   resource VARCHAR(1024)
# );
#
# Notes:
# 1. Missing category update timestamps are set to current time
#
# * "quick" does not refer to run time. Expect a long wait for a full
# structure file to be loaded into the database!
#
# History
# 
# 2004 Feb 20 - v1.1 Added patches from will@hellacool.co.uk that simplify
#                    the DBI code by using placeholders.
#
#==========================================================================
# Copyright (c) 2003-2004 by R. Steven Rainwater <steve@ncc.com>
# 
# http://www.ncc.com/freeware/
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#==========================================================================
#
use strict;
use bytes;
use XML::Parser;
use DBI;

$| =1;

# Globals containing the current category information
my $topic = '';
my $catid = '';
my $title = '';
my $lastupdate = '';
my $description = '';
my @resource = ();

my $records = 0;

# Sort of an XML tag stack thingy
my @pendingTag = ();

# connect to database
my $dbh = DBI->connect("dbi:Pg:dbname=name;host=db.host.com","username","password")
or die "Can't connect to database: $DBI::errstr\n";

# purge any existing structure data
$dbh->do("DELETE FROM structure") or print "\nDelete Failed: $DBI::errstr\n";
$dbh->do("DELETE FROM resources") or print "\nDelete Failed: $DBI::errstr\n";

# Create the statement handles
my $res_sth = $dbh->prepare("INSERT INTO resources VALUES (?, ?, ?)");
my $str_sth = $dbh->prepare("INSERT INTO structure VALUES (?, ?, ?, ?, ?)");

# initialize XML parser
my $p1 = new XML::Parser(Style => 'Stream',
                      ErrorContext => 2,
		      ProtocolEncoding => 'UTF-8',
		      NoExpand => 1,
		      ParseParamEnt => 0);

# parse the file
$p1->parsefile('structure.clean');
$dbh->disconnect or die "Disconnect failed: $DBI::errstr\n";
exit;

# XML processing subs

sub StartDocument {
  print "\nLoading structure records\n";
  return;
}

sub StartTag {
  m/^<(\S+)[\s>].*/;
  my $tag = $1;
  unshift @pendingTag, $tag;
  if($tag eq 'Topic') {
    m/.*"(.+)">$/;
    $topic = $1;
    $catid = '';
    $title = '';
    $lastupdate = '';
    $description = '';
    @resource = ();
  }
  elsif($pendingTag[1] eq 'Topic') {
    if(m/.*"(.+)">$/) { push @resource, [($tag, $1)]; }
  }
  elsif($tag eq 'Alias' || $pendingTag[1] eq 'Alias' || $tag eq 'RDF') { return; }
  else { print "Warning: Unhandled Tag [$tag] (Pending [$pendingTag[1]])\n"; }
}

sub EndTag {
  m/^<\/(\S+)>$/;
  my $tag = $1;
  shift @pendingTag;
  if($tag eq 'Topic') {
    # insert this topic block into database
    unless($lastupdate) { $lastupdate = 'now'; }
    $str_sth->execute($catid, $topic, $title, $description, $lastupdate)
    or print 
      "\n------\n",
      "Structure Insert Failed: $DBI::errstr\n",
      "Topic: $topic\n",
      "Catid: $catid\n",
      "Title: $title\n";
    foreach(@resource) {
      $res_sth->execute($catid, @{$_}[0], @{$_}[1])
      or print "\n------\nResource Insert Failed: $DBI::errstr\n@{$_}[0]\n";
    }
    $records++;
    unless($records % 1000) { print "Record: $records\n"; }
  }
}

sub Text {
  if($pendingTag[0] eq 'catid') { $catid = $_; }
  elsif($pendingTag[0] eq 'd:Title') { $title = $_; }
  elsif($pendingTag[0] eq 'lastUpdate') { $lastupdate = $_; }
  elsif($pendingTag[0] eq 'd:Description') { $description = $_; }
}

sub PI {
  return;
}

sub EndDocument {
  print "$records loaded\n";
  return;
}
